/*******************************************************************
    Control an LED strip using Inline Keyboard on Telegram

    By Brian Lough
    YouTube: https://www.youtube.com/brianlough
    Tindie: https://www.tindie.com/stores/brianlough/
    Twitter: https://twitter.com/witnessmenow

    Updated by Riccardo Grosso, 05/30/2024
    https://www.ethicaldiy.org
 *******************************************************************/
#include <ESP8266WiFi.h>
#include <WiFiClientSecure.h>

#include <UniversalTelegramBot.h>
// Library for interacting with the Telegram API
// Search for "Telegegram" in the Library manager and install
// The universal Telegram library
// https://github.com/witnessmenow/Universal-Arduino-Telegram-Bot


#include <ArduinoJson.h>
// Library used for parsing Json from the API responses
// NOTE: There is a breaking change in the 6.x.x version,
// install the 5.x.x version instead
// Search for "Arduino Json" in the Arduino Library manager
// https://github.com/bblanchon/ArduinoJson

//------- Replace the following! ------

char ssid[] = "inserisci il nome della tua rete WiFi";
char password[] = "inserisci la password della rete WiFi";
// Telegram BOT Token (Get from Botfather)
#define TELEGRAM_BOT_TOKEN "inserisci il tuo toked (codice alfanumerico lungo)"


//------- ---------------------- ------


// This is the Wifi client that supports HTTPS
WiFiClientSecure client;
UniversalTelegramBot bot(TELEGRAM_BOT_TOKEN, client);

#define LED_PIN1 2 // Same as D4 for Wemos
int ledStatus1 = 0;
#define LED_PIN2 0
int ledStatus2 = 0;

/* I led e i relè abitualmente si attivano con ledon e si spengono con ledoff
   Ma in alcuni apparecchi commerciali, i comandi sono invertiti
   se lanciando il programma i led/relè si accendono quando si lancia il comando di spegnimento
   e si spengono quando dovrebbero accendersi, allora commentare le due linee seguenti */

int changeVal = 1;   //if cangeVal = "1", relay1on = HIGH; relay1off = LOW; else relay1on = LOW; relay1off = HIGH 
int led1on;
int led1off;
int led2on;
int led2off;

int delayBetweenChecks1 = 1000;
int delayBetweenChecks2 = 1000;
unsigned long lastTimeChecked1;   //last time messages' scan has been done
unsigned long lastTimeChecked2;   //last time messages' scan has been done

unsigned long lightTimerExpires1;
boolean lightTimerActive1 = false;
unsigned long lightTimerExpires2;
boolean lightTimerActive2 = false;

void setup() {
  Serial.begin(115200);

  if (changeVal == 1){ 
    led1on = 1;      
    led1off = 0;
    led2on = 1;
    led2off = 0;
  }    
  else{                
    led1on = 0;
    led1off = 1;  
    led2on = 0;
    led2off = 1; 
  }

  // Set WiFi to station mode and disconnect from an AP if it was Previously connected
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);

  pinMode(2, OUTPUT);
  digitalWrite(2, led1off);
  pinMode(0, OUTPUT);
  digitalWrite(0, led2off);

  // attempt to connect to Wifi network:
  Serial.print("Connecting Wifi: ");
  Serial.println(ssid);
  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  // Only required on 2.5 Beta
  client.setInsecure();

  
  // longPoll keeps the request to Telegram open for the given amount of seconds if there are no messages
  // This hugely improves response time of the bot, but is only really suitable for projects
  // where the the initial interaction comes from Telegram as the requests will block the loop for
  // the length of the long poll
  bot.longPoll = 60;
}

void handleNewMessages(int numNewMessages) {

  for (int i = 0; i < numNewMessages; i++) {

    // If the type is a "callback_query", a inline keyboard button was pressed
    if (bot.messages[i].type ==  F("callback_query")) {
      String text = bot.messages[i].text;
      String chat_id = bot.messages[i].chat_id;
      Serial.print("Call back button pressed with text: ");
      Serial.println(text);


      if (text == F("ON")) {
        digitalWrite(LED_PIN1, led1on);
        ledStatus1 = 1;
        bot.sendMessage(chat_id, "Il led/relè n.1 è attivo", "");
        digitalWrite(LED_PIN2, led2on);
        ledStatus2 = 1;
        bot.sendMessage(chat_id, "Il led/relè n.2 è attivo", "");
      } else if (text == F("OFF")) {
        digitalWrite(LED_PIN1, led1off);
        ledStatus1 = 0;
        bot.sendMessage(chat_id, "Il led/relè n.1 è spento", "");
        digitalWrite(LED_PIN2, led2off);
        ledStatus2 = 0;
        bot.sendMessage(chat_id, "Il led/relè n.2 è spento", "");
      }
      if (text == F("ON1")) {
        digitalWrite(LED_PIN1, led1on);
        ledStatus1 = 1;
        bot.sendMessage(chat_id, "Il led/relè n.1 è attivo", "");
      } else if (text == F("OFF1")) {
        digitalWrite(LED_PIN1, led1off);
        ledStatus1 = 0;
        bot.sendMessage(chat_id, "Il led/relè n.1 è spento", "");
      } else if (text.startsWith("1TIME")) {
        text.replace("1TIME", "");
        int timeRequested1 = text.toInt();    
        digitalWrite(LED_PIN1, led1on);
        ledStatus1 = 1;
        bot.sendMessage(chat_id, "Il led/relè n.1 è stato attivato a tempo", "");
        lightTimerActive1 = true;
        lightTimerExpires1 = millis() + (timeRequested1 * 1000 * 60);
      }

      if (text == F("ON2")) {
        digitalWrite(LED_PIN2, led2on);
        ledStatus2 = 1;
        bot.sendMessage(chat_id, "Il led/relè n.2 è attivo", "");
      } else if (text == F("OFF2")) {
        digitalWrite(LED_PIN2, led2off);
        ledStatus2 = 0;
        bot.sendMessage(chat_id, "Il led/relè n.2 è spento", "");
      } else if (text.startsWith("2TIME")) {
        text.replace("2TIME", "");
        int timeRequested2 = text.toInt();    
        digitalWrite(LED_PIN2, led2on);
        ledStatus2 = 1;
        bot.sendMessage(chat_id, "Il led/relè n.2 è stato attivato a tempo", "");
        lightTimerActive2 = true;
        lightTimerExpires2 = millis() + (timeRequested2 * 1000 * 60);
}

      
    } else {
      String chat_id = String(bot.messages[i].chat_id);
      String text = bot.messages[i].text;



     if (text == "/status")
    {
      if (ledStatus1 == 1)
      {
        bot.sendMessage(chat_id, "Il led/relè n.1 è attivo", "");
      }
      if (ledStatus1 == 0)
      {
        bot.sendMessage(chat_id, "Il led/relè n.1 è spento", "");
      }
      if (ledStatus1 == 2)
      {
        bot.sendMessage(chat_id, "Il led/relè n.1 è stato spento automaticamente", "");
      }

      if (ledStatus2 == 1)
      {
        bot.sendMessage(chat_id, "Il led/relè n.2 è attivo", "");
      }
      if (ledStatus2 == 0)
      {
        bot.sendMessage(chat_id, "Il led/relè n.2 è spento", "");
      }
            if (ledStatus2 == 2)
      {
        bot.sendMessage(chat_id, "Il led/relè n.2 è stato spento automaticamente", "");
      }
    }     

      if (text == F("/options")) {

        // Keyboard Json is an array of arrays.
        // The size of the main array is how many row options the keyboard has
        // The size of the sub arrays is how many coloums that row has

        // "The Text" property is what shows up in the keyboard
        // The "callback_data" property is the text that gets sent when pressed  
        
        String keyboardJson = F("[[{ \"text\" : \"ON\", \"callback_data\" : \"ON\" },{ \"text\" : \"OFF\", \"callback_data\" : \"OFF\" },{ \"text\" : \"ON_1\", \"callback_data\" : \"ON1\" },{ \"text\" : \"OFF_1\", \"callback_data\" : \"OFF1\" }, { \"text\" : \"ON_2\", \"callback_data\" : \"ON2\" },{ \"text\" : \"OFF_2\", \"callback_data\" : \"OFF2\" } ],");
        keyboardJson += F("[{ \"text\" : \"15_1\", \"callback_data\" : \"1TIME15\" }, { \"text\" : \"30_1\", \"callback_data\" : \"1TIME30\" }, { \"text\" : \"60_1\", \"callback_data\" : \"1TIME60\" },{ \"text\" : \"45_2\", \"callback_data\" : \"2TIME45\" }, { \"text\" : \"90_2\", \"callback_data\" : \"2TIME90\" }, { \"text\" : \"120_2\", \"callback_data\" : \"2TIME120\"  }]]");
        bot.sendMessageWithInlineKeyboard(chat_id, "Comandi disponibili", "", keyboardJson);
      }



      // When a user first uses a bot they will send a "/start" command
      // So this is a good place to let the users know what commands are available
      if (text == F("/start")) {

        bot.sendMessage(chat_id, "/options : attiva la tastiera virtuale\n", "Markdown");
        bot.sendMessage(chat_id, "/status : mostra lo stato dei led/relè\n", "Markdown");
        String keyboardJson = F("[[{ \"text\" : \"ON\", \"callback_data\" : \"ON\" },{ \"text\" : \"OFF\", \"callback_data\" : \"OFF\" },{ \"text\" : \"ON_1\", \"callback_data\" : \"ON1\" },{ \"text\" : \"OFF_1\", \"callback_data\" : \"OFF1\" }, { \"text\" : \"ON_2\", \"callback_data\" : \"ON2\" },{ \"text\" : \"OFF_2\", \"callback_data\" : \"OFF2\" } ],");
        keyboardJson += F("[{ \"text\" : \"15_1\", \"callback_data\" : \"1TIME15\" }, { \"text\" : \"30_1\", \"callback_data\" : \"1TIME30\" }, { \"text\" : \"60_1\", \"callback_data\" : \"1TIME60\" },{ \"text\" : \"45_2\", \"callback_data\" : \"2TIME45\" }, { \"text\" : \"90_2\", \"callback_data\" : \"2TIME90\" }, { \"text\" : \"120_2\", \"callback_data\" : \"2TIME120\"  }]]");
        bot.sendMessageWithInlineKeyboard(chat_id, "Comandi disponibili", "", keyboardJson);

      }
    }
  }
}

void loop() {
  if (millis() > lastTimeChecked1 + delayBetweenChecks1)  {

    // getUpdates returns 1 if there is a new message from Telegram
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    if (numNewMessages) {
      Serial.println("got response");
      handleNewMessages(numNewMessages);
    }

    lastTimeChecked1 = millis();

    if (lightTimerActive1 && millis() > lightTimerExpires1) {
      lightTimerActive1 = false;
      digitalWrite(LED_PIN1, led1off);
      ledStatus1 = 2;
    }
  }

    if (millis() > lastTimeChecked2 + delayBetweenChecks2)  {

    // getUpdates returns 1 if there is a new message from Telegram
    int numNewMessages = bot.getUpdates(bot.last_message_received + 1);

    if (numNewMessages) {
      Serial.println("got response");
      handleNewMessages(numNewMessages);
    }

    lastTimeChecked2 = millis();

    if (lightTimerActive2 && millis() > lightTimerExpires2) {
      lightTimerActive2 = false;
      digitalWrite(LED_PIN2, led2off);
      ledStatus2 = 2;
    }
  }
}
